==> gcdome-2.2.0.tar.gz: gcdome-2.2.0/gcdome/gcstrut.c <==
/*
gcppm.c Version 2.2.0. JPEG program for great circle geodesic dome
Copyright (C) 2001  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <asm/errno.h>
#include <zlib.h>
#include <math.h>

#define PI M_PI
#define MAXFREQ 32
#define NINETY 90.0
#define MINNUM 0.00000000001
#define NEGZERO(x) { if (x > -MINNUM && x < MINNUM) { x = zero; } }
#define SZ 480.0
#define XSZ 480
#define YSZ 480
#define BLACK 1
#define RED 2

int freq,diam,strut;
double pi,lon1,lon2,lon2d;
double lon,lond;
double dblfreq,dbldiam,radius;
double height,heightd,rheight,rheightd;
double deg2rad,rad2deg,ninety,zero;
double *phi,*theta,*curphi,*curtheta;
double arclen(),gclat();
char comma[8],newline[8];

char mtx[YSZ][XSZ];

void putstx(pgm)
   {
   fprintf(stderr,"Usage: %s <frequency> <diameter>\n", pgm);
   fprintf(stderr,"   Where frequency is 1,2,3...\n", pgm);
   exit(1);
   } /* putstx */

void bldhead(freq,diam)
int freq,diam;
   {
   printf("<HTML>\n");
   printf("<HEAD>\n");
   printf("<TITLE>\n");
   printf("Great Circle Geodesic Dome:\n");
   printf("Frequency %d Diameter %d\n", freq, diam);
   printf("</TITLE>\n");
   printf("</HEAD>\n");
   printf("<BODY BGCOLOR = \"FFFFFF\">\n");
   printf("<H1 ALIGN = CENTER>\n");
   printf("Great Circle Geodesic Dome\n");
   printf("<BR>\n");
   printf("Frequency %d\n", freq);
   printf("&nbsp; &nbsp; Diameter %d\n", diam);
   printf("</H1>\n");
   } /* bldhead */

void putimg(hndl,str,len)
gzFile hndl;
unsigned char *str;
int len;
   {
   int wrtlen,tot;
   tot = 0;
   while (tot < len)
      {
      wrtlen = gzwrite(hndl,str,len);
      if (wrtlen < 1)
         {
         fprintf(stderr,"Write error %d\n", errno);
         perror("Write error");
         exit(1);
         } /* write err */
      tot += wrtlen;
      } /* write loop */
   } /* putimg */

void wrtppm(hndl)
gzFile hndl;
   {
   int len,x,y;
   unsigned char str[32];
   putimg(hndl,"P6\n",3);
   sprintf(str,"%d %d\n%d\n",
      XSZ, YSZ, 255);
   len = strlen(str);
   putimg(hndl,str,len);
   for (y=0;y<YSZ;y++)
      {
      for (x=0;x<XSZ;x++)
         {
         if (mtx[y][x] == BLACK)
            {
            str[0] = str[1] = str[2] = 0;
            } /* if black */
         else if (mtx[y][x] == RED)
            {
            str[0] = 255;
            str[1] = str[2] = 0;
            } /* if red */
         else
            {
            str[0] = str[1] = str[2] = 255;
            } /* else white */
         putimg(hndl,str,3);
         } /* for each col in mtx */
      } /* for each row in mtx */
   } /* wrtppm */

void drawline(dblx1,dbly1,dblx2,dbly2,color)
   double dblx1,dbly1,dblx2,dbly2;
   int color;
   {
   int x1,y1,x2,y2;
   int x,y,dx,dy,xdlta,ydlta,absxdlta,absydlta;
   int xchg,ychg,xlim,ylim;
   x1 = (int) dblx1;
   y1 = (int) dbly1;
   x2 = (int) dblx2;
   y2 = (int) dbly2;
   if (x1 < 0 || x1 > XSZ)
      {
      fprintf(stderr,"x1 %d out of range\n", x1);
      return;
      } /* x1 err */
   if (y1 < 0 || y1 > YSZ)
      {
      fprintf(stderr,"y1 %d out of range\n", y1);
      return;
      } /* y1 err */
   if (x2 < 0 || x2 > XSZ)
      {
      fprintf(stderr,"x2 %d out of range\n", x2);
      return;
      } /* x2 err */
   if (y2 < 0 || y2 > YSZ)
      {
      fprintf(stderr,"y2 %d out of range\n", y2);
      return;
      } /* y2 err */
   if (color == BLACK || color == RED);
   else
      {
      fprintf(stderr,"Invalid color %d\n", color);
      return;
      } /* color err */
   if (x1 < x2)
      {
      dx = x2 - x1;
      dy = y2 - y1;
      if (dy) xdlta = (dx << 8) / dy;
      else xdlta = 256;
      ydlta = (dy << 8) / dx;
      x = x1;
      y = y1;
      xlim = x2;
      ylim = y2;
      } /* if slope to right */
   else if (x1 > x2)
      {
      dx = x1 - x2;
      dy = y1 - y2;
      if (dy) xdlta = (dx << 8) / dy;
      else xdlta = 256;
      ydlta = (dy << 8) / dx;
      x = x2;
      y = y2;
      xlim = x1;
      ylim = y1;
      } /* if slope to left */
   else
      {
      x = x1;
      y = y1;
      if (y2 > y1)
         {
         y = y1;
         ylim = y2;
         } /* if y2 > y1 */
      else
         {
         y = y2;
         ylim = y1;
         } /* else y1 > y2 */
      while (y < ylim)
         mtx[y++][x] = color;
      return;
      } /* vertical line */
   if (xdlta < 0) absxdlta = -xdlta;
   else absxdlta = xdlta;
   if (ydlta < 0) absydlta = -ydlta;
   else absydlta = ydlta;
   xchg = 0;
   ychg = 0;
   while (x < xlim)
      {
      xchg += absxdlta;
      if (xchg > 256)
         {
         if (x <= xlim) x++;
         xchg -= 256;
         } /* if whole number */
      ychg += absydlta;
      if (ychg > 128)
         {
         if (ydlta < 0)
            {
            if (y >= ylim) y--;
            } /* if negative slope */
         else
            {
            if (y <= ylim) y++;
            } /* if positive slope */
         ychg -= 256;
         } /* if whole number */
      mtx[y][x] = color;
      } /* draw line */
   } /* drawline */

void putppm(freq,phi1,theta1,phi2,theta2)
double phi1,theta1,phi2,theta2;
   {
   int black,red,rslt;
   double tmpphi,tmptheta,x1,y1,z1,x2,y2,z2;
   double toprow,dx,dy,dz,dxsq,dysq,dzsq;
   double strutlen;
   char fname[128];
   gzFile hndl;

   black = 1;
   red   = 2;
   toprow = SZ + 32;
   tmpphi   = phi1   * deg2rad;
   tmptheta = theta1 * deg2rad;
   x1 = sin(tmptheta) * cos(tmpphi) * SZ;
   y1 = sin(tmptheta) * sin(tmpphi) * SZ + 32;
   z1 = toprow - cos(tmptheta) * SZ;
   NEGZERO(x1);
   NEGZERO(y1);
   NEGZERO(z1);
   tmpphi   = phi2   * deg2rad;
   tmptheta = theta2 * deg2rad;
   x2 = sin(tmptheta) * cos(tmpphi) * SZ;
   y2 = sin(tmptheta) * sin(tmpphi) * SZ + 32;
   z2 = toprow - cos(tmptheta) * SZ;
   NEGZERO(x2);
   NEGZERO(y2);
   NEGZERO(z2);
   drawline(y1, z1, y2, z2, red);	
   sprintf(fname,"f%d%02d.ppm.gz", freq, ++strut);
   /* wb9 for maximum compression */
   hndl = gzopen(fname,"wb9");
   if (hndl == NULL)
      {
      fprintf(stderr,"Open error");
      perror("Open error");
      exit(1);
      } /* open err */
   wrtppm(hndl);
   rslt = gzclose(hndl);
   if (rslt < 0)
      {
      fprintf(stderr,"Close error");
      perror("Close error");
      exit(1);
      } /* close err */
   drawline(y1, z1, y2, z2, black);	
   tmpphi   = phi1   * deg2rad;
   tmptheta = theta1 * deg2rad;
   x1 = sin(tmptheta) * cos(tmpphi) * radius;
   y1 = sin(tmptheta) * sin(tmpphi) * radius;
   z1 = cos(tmptheta) * radius;
   NEGZERO(x1);
   NEGZERO(y1);
   NEGZERO(z1);
   tmpphi   = phi2   * deg2rad;
   tmptheta = theta2 * deg2rad;
   x2 = sin(tmptheta) * cos(tmpphi) * radius;
   y2 = sin(tmptheta) * sin(tmpphi) * radius;
   z2 = cos(tmptheta) * radius;
   NEGZERO(x2);
   NEGZERO(y2);
   NEGZERO(z2);
   dx = x2 - x1;
   dy = y2 - y1;
   dz = z2 - z1;
   dxsq = dx * dx;
   dysq = dy * dy;
   dzsq = dz * dz;
   strutlen = sqrt(dxsq + dysq + dzsq);
   printf("<H2>\n");
   printf("<A HREF=\"f%d%02d.gif\">\n", freq, strut);
   printf("Strut %d </A>\n", strut);
   printf("&nbsp; &nbsp; Length %.13f\n", strutlen);
   printf("</H2>\n");

   printf("<TABLE BORDER=1 ALIGN=CENTER>\n");
   printf("<TR>\n");
   printf("<TH>\n");
   printf("</TH>\n");

   printf("<TH>\n");
   printf("Horiz. Angle\n");
   printf("</TH>\n");

   printf("<TH>\n");
   printf("Vert. Angle\n");
   printf("</TH>\n");
   printf("</TR>\n");

   printf("<TR>\n");
   printf("<TD>\n");
   printf("From\n");
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", phi1);
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", theta1);
   printf("</TD>\n");
   printf("</TR>\n");

   printf("<TR>\n");
   printf("<TD>\n");
   printf("To\n");
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", phi2);
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", theta2);
   printf("</TD>\n");
   printf("</TR>\n");
   printf("</TABLE>\n");

   printf("<P>\n");
   if (freq == 2)
      {
      printf("<IMAGE SRC=\"f%d%02d.gif\">\n", freq, strut);
      printf("<P>\n");
      }
   } /* putppm */

void plotppm(freq,topphi,toptheta,
	lftphi,lfttheta,rgtphi,rgttheta)
int freq;
double topphi,toptheta,lftphi,lfttheta,rgtphi,rgttheta;
   {
   putppm(freq,topphi,toptheta,lftphi,lfttheta);
   putppm(freq,topphi,toptheta,rgtphi,rgttheta);
   putppm(freq,lftphi,lfttheta,rgtphi,rgttheta);
   } /* plotpt */


void makeppm(freq)
int freq;
   {
   int i,segs,rownum,strut;
   double lonseg;
   double seglon,seglond,dblsegs;
   double seglend;
   double lat,latd,lat1,lat1d,lat2,lat2d;

   dblsegs = dblfreq;
   segs = freq;  /* segments per row */
   seglend = lon2d / dblfreq;
   lat1d = ninety - heightd;
   lat1 = lat1d * deg2rad;
   rownum = strut = 0;
   while (floor(lat1d) <= ninety)
      {
      lat2 = lat1;
      lat2d = lat2 * rad2deg;
      seglon = curphi[0] = zero;
      curtheta[0] = ninety - lat2d;
      for (i=0;i<segs;i++)
         {
         lonseg = seglend * deg2rad;
         seglon += lonseg;
         seglond = seglon * rad2deg;
         /* get mid-course great circle */
         /* latitude in radians */
         lat = (double) gclat(lat1,lon1,lat2,lon2,seglon);
         latd = lat * rad2deg;
         curphi[i+1] = seglond;
         curtheta[i+1] = ninety - latd;
         if (rownum > 0)
            {
            plotppm(freq,curphi[i],curtheta[i],
               phi[i],theta[i],phi[i+1],theta[i+1]);
            }
         } /* for each segment of the great circle */
      if (rownum > 0 && rownum < freq + 1)
         {
         plotppm(freq,curphi[i],curtheta[i],
            phi[i],theta[i],phi[i+1],theta[i+1]);
         }
      for (i=0;i<=segs;i++)
         {
         phi[i] = curphi[i];
         theta[i] = curtheta[i];
         }
      lat1 += rheight;
      lat1d += rheightd;
      segs--;
      dblsegs--;
      seglend = lon2d / dblsegs;
      rownum++;
      } /* for each row */
   } /* makeppm */

int main(argc,argv)
int argc;
char **argv;
   {
   if (argc != 3) putstx(*argv);
   freq = atoi(*(argv+1)); 
   if (freq < 1) putstx(*argv);
   else if (freq > MAXFREQ)
      {
      fprintf(stderr,"Frequency %d too large\n", freq);
      fprintf(stderr,"Maximum frequency is %d\n", MAXFREQ);
      putstx(*argv);
      }
   diam = atoi(*(argv+2));
   if (diam < 1) putstx(*argv);
   dblfreq  = (double) freq;
   dbldiam  = (double) diam;
   radius   = dbldiam / 2.0;
   phi      = (double *) malloc(sizeof(double)*(freq+32));
   theta    = (double *) malloc(sizeof(double)*(freq+32));
   curphi   = (double *) malloc(sizeof(double)*(freq+32));
   curtheta = (double *) malloc(sizeof(double)*(freq+32));
   strcpy(comma,", ");
   strcpy(newline,"\n");
   zero = 0.0;
   ninety = NINETY;
   pi = PI;
   deg2rad = pi / 180.0;
   rad2deg = 180.0 / pi;
   lon1 = zero;
   lon2d = 72.0;
   lon2 = lon2d * deg2rad;
   height = (double) arclen();
   heightd = height * rad2deg;
   rheight = height / dblfreq; /* row height in radians */
   rheightd = rheight * rad2deg;
   bldhead(freq,diam);
   makeppm(freq);
   printf("</BODY>\n");
   printf("</HTML>\n");
   return(0);
   } /* main */

==> gcdome-2.3.0.tar.gz: gcdome-2.3.0/gcdome/gcstrut.c <==
/*
gcppm.c Version 2.3.0. JPEG program for great circle geodesic dome
Copyright (C) 2001-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <asm/errno.h>
#include <zlib.h>
#include <math.h>

#define PI M_PI
#define MAXFREQ 32
#define NINETY 90.0
#define MINNUM 0.00000000001
#define NEGZERO(x) { if (x > -MINNUM && x < MINNUM) { x = zero; } }
#define SZ 480.0
#define XSZ 480
#define YSZ 480
#define BLACK 1
#define RED 2

int freq,diam,strut;
double pi,lon1,lon2,lon2d;
double lon,lond;
double dblfreq,dbldiam,radius;
double height,heightd,rheight,rheightd;
double deg2rad,rad2deg,ninety,zero;
double *phi,*theta,*curphi,*curtheta;
double arclen(),gclat();
char comma[8],newline[8];

char mtx[YSZ][XSZ];

void putstx(char *pgm)
   {
   fprintf(stderr,"Usage: %s <frequency> <diameter>\n", pgm);
   fprintf(stderr,"   Where frequency is 1,2,3...\n", pgm);
   exit(1);
   } /* putstx */

void bldhead(freq,diam)
int freq,diam;
   {
   printf("<HTML>\n");
   printf("<HEAD>\n");
   printf("<TITLE>\n");
   printf("Great Circle Geodesic Dome:\n");
   printf("Frequency %d Diameter %d\n", freq, diam);
   printf("</TITLE>\n");
   printf("</HEAD>\n");
   printf("<BODY BGCOLOR = \"FFFFFF\">\n");
   printf("<H1 ALIGN = CENTER>\n");
   printf("Great Circle Geodesic Dome\n");
   printf("<BR>\n");
   printf("Frequency %d\n", freq);
   printf("&nbsp; &nbsp; Diameter %d\n", diam);
   printf("</H1>\n");
   } /* bldhead */

void putimg(hndl,str,len)
gzFile hndl;
unsigned char *str;
int len;
   {
   int wrtlen,tot;
   tot = 0;
   while (tot < len)
      {
      wrtlen = gzwrite(hndl,str,len);
      if (wrtlen < 1)
         {
         fprintf(stderr,"Write error %d\n", errno);
         perror("Write error");
         exit(1);
         } /* write err */
      tot += wrtlen;
      } /* write loop */
   } /* putimg */

void wrtppm(hndl)
gzFile hndl;
   {
   int len,x,y;
   unsigned char str[32];
   putimg(hndl,"P6\n",3);
   sprintf(str,"%d %d\n%d\n",
      XSZ, YSZ, 255);
   len = strlen(str);
   putimg(hndl,str,len);
   for (y=0;y<YSZ;y++)
      {
      for (x=0;x<XSZ;x++)
         {
         if (mtx[y][x] == BLACK)
            {
            str[0] = str[1] = str[2] = 0;
            } /* if black */
         else if (mtx[y][x] == RED)
            {
            str[0] = 255;
            str[1] = str[2] = 0;
            } /* if red */
         else
            {
            str[0] = str[1] = str[2] = 255;
            } /* else white */
         putimg(hndl,str,3);
         } /* for each col in mtx */
      } /* for each row in mtx */
   } /* wrtppm */

void drawline(dblx1,dbly1,dblx2,dbly2,color)
   double dblx1,dbly1,dblx2,dbly2;
   int color;
   {
   int x1,y1,x2,y2;
   int x,y,dx,dy,xdlta,ydlta,absxdlta,absydlta;
   int xchg,ychg,xlim,ylim;
   x1 = (int) dblx1;
   y1 = (int) dbly1;
   x2 = (int) dblx2;
   y2 = (int) dbly2;
   if (x1 < 0 || x1 > XSZ)
      {
      fprintf(stderr,"x1 %d out of range\n", x1);
      return;
      } /* x1 err */
   if (y1 < 0 || y1 > YSZ)
      {
      fprintf(stderr,"y1 %d out of range\n", y1);
      return;
      } /* y1 err */
   if (x2 < 0 || x2 > XSZ)
      {
      fprintf(stderr,"x2 %d out of range\n", x2);
      return;
      } /* x2 err */
   if (y2 < 0 || y2 > YSZ)
      {
      fprintf(stderr,"y2 %d out of range\n", y2);
      return;
      } /* y2 err */
   if (color == BLACK || color == RED);
   else
      {
      fprintf(stderr,"Invalid color %d\n", color);
      return;
      } /* color err */
   if (x1 < x2)
      {
      dx = x2 - x1;
      dy = y2 - y1;
      if (dy) xdlta = (dx << 8) / dy;
      else xdlta = 256;
      ydlta = (dy << 8) / dx;
      x = x1;
      y = y1;
      xlim = x2;
      ylim = y2;
      } /* if slope to right */
   else if (x1 > x2)
      {
      dx = x1 - x2;
      dy = y1 - y2;
      if (dy) xdlta = (dx << 8) / dy;
      else xdlta = 256;
      ydlta = (dy << 8) / dx;
      x = x2;
      y = y2;
      xlim = x1;
      ylim = y1;
      } /* if slope to left */
   else
      {
      x = x1;
      y = y1;
      if (y2 > y1)
         {
         y = y1;
         ylim = y2;
         } /* if y2 > y1 */
      else
         {
         y = y2;
         ylim = y1;
         } /* else y1 > y2 */
      while (y < ylim)
         mtx[y++][x] = color;
      return;
      } /* vertical line */
   if (xdlta < 0) absxdlta = -xdlta;
   else absxdlta = xdlta;
   if (ydlta < 0) absydlta = -ydlta;
   else absydlta = ydlta;
   xchg = 0;
   ychg = 0;
   while (x < xlim)
      {
      xchg += absxdlta;
      if (xchg > 256)
         {
         if (x <= xlim) x++;
         xchg -= 256;
         } /* if whole number */
      ychg += absydlta;
      if (ychg > 128)
         {
         if (ydlta < 0)
            {
            if (y >= ylim) y--;
            } /* if negative slope */
         else
            {
            if (y <= ylim) y++;
            } /* if positive slope */
         ychg -= 256;
         } /* if whole number */
      mtx[y][x] = color;
      } /* draw line */
   } /* drawline */

void putppm(freq,phi1,theta1,phi2,theta2)
double phi1,theta1,phi2,theta2;
   {
   int black,red,rslt;
   double tmpphi,tmptheta,x1,y1,z1,x2,y2,z2;
   double toprow,dx,dy,dz,dxsq,dysq,dzsq;
   double strutlen;
   char fname[128];
   gzFile hndl;

   black = 1;
   red   = 2;
   toprow = SZ + 32;
   tmpphi   = phi1   * deg2rad;
   tmptheta = theta1 * deg2rad;
   x1 = sin(tmptheta) * cos(tmpphi) * SZ;
   y1 = sin(tmptheta) * sin(tmpphi) * SZ + 32;
   z1 = toprow - cos(tmptheta) * SZ;
   NEGZERO(x1);
   NEGZERO(y1);
   NEGZERO(z1);
   tmpphi   = phi2   * deg2rad;
   tmptheta = theta2 * deg2rad;
   x2 = sin(tmptheta) * cos(tmpphi) * SZ;
   y2 = sin(tmptheta) * sin(tmpphi) * SZ + 32;
   z2 = toprow - cos(tmptheta) * SZ;
   NEGZERO(x2);
   NEGZERO(y2);
   NEGZERO(z2);
   drawline(y1, z1, y2, z2, red);	
   sprintf(fname,"f%d%02d.ppm.gz", freq, ++strut);
   /* wb9 for maximum compression */
   hndl = gzopen(fname,"wb9");
   if (hndl == NULL)
      {
      fprintf(stderr,"Open error");
      perror("Open error");
      exit(1);
      } /* open err */
   wrtppm(hndl);
   rslt = gzclose(hndl);
   if (rslt < 0)
      {
      fprintf(stderr,"Close error");
      perror("Close error");
      exit(1);
      } /* close err */
   drawline(y1, z1, y2, z2, black);	
   tmpphi   = phi1   * deg2rad;
   tmptheta = theta1 * deg2rad;
   x1 = sin(tmptheta) * cos(tmpphi) * radius;
   y1 = sin(tmptheta) * sin(tmpphi) * radius;
   z1 = cos(tmptheta) * radius;
   NEGZERO(x1);
   NEGZERO(y1);
   NEGZERO(z1);
   tmpphi   = phi2   * deg2rad;
   tmptheta = theta2 * deg2rad;
   x2 = sin(tmptheta) * cos(tmpphi) * radius;
   y2 = sin(tmptheta) * sin(tmpphi) * radius;
   z2 = cos(tmptheta) * radius;
   NEGZERO(x2);
   NEGZERO(y2);
   NEGZERO(z2);
   dx = x2 - x1;
   dy = y2 - y1;
   dz = z2 - z1;
   dxsq = dx * dx;
   dysq = dy * dy;
   dzsq = dz * dz;
   strutlen = sqrt(dxsq + dysq + dzsq);
   printf("<H2>\n");
   printf("<A HREF=\"f%d%02d.gif\">\n", freq, strut);
   printf("Strut %d </A>\n", strut);
   printf("&nbsp; &nbsp; Length %.13f\n", strutlen);
   printf("</H2>\n");

   printf("<A IMG SRC=\"f%d%02d.gif\">\n", freq, strut);

   printf("<TABLE BORDER=1 ALIGN=CENTER>\n");
   printf("<TR>\n");
   printf("<TH>\n");
   printf("</TH>\n");

   printf("<TH>\n");
   printf("Horiz. Angle\n");
   printf("</TH>\n");

   printf("<TH>\n");
   printf("Vert. Angle\n");
   printf("</TH>\n");
   printf("</TR>\n");

   printf("<TR>\n");
   printf("<TD>\n");
   printf("From\n");
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", phi1);
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", theta1);
   printf("</TD>\n");
   printf("</TR>\n");

   printf("<TR>\n");
   printf("<TD>\n");
   printf("To\n");
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", phi2);
   printf("</TD>\n");

   printf("<TD>\n");
   printf("%.13f\n", theta2);
   printf("</TD>\n");
   printf("</TR>\n");
   printf("</TABLE>\n");

   printf("<P>\n");
   if (freq == 2)
      {
      printf("<IMAGE SRC=\"f%d%02d.gif\">\n", freq, strut);
      printf("<P>\n");
      }
   } /* putppm */

void plotppm(freq,topphi,toptheta,
	lftphi,lfttheta,rgtphi,rgttheta)
int freq;
double topphi,toptheta,lftphi,lfttheta,rgtphi,rgttheta;
   {
   putppm(freq,topphi,toptheta,lftphi,lfttheta);
   putppm(freq,topphi,toptheta,rgtphi,rgttheta);
   putppm(freq,lftphi,lfttheta,rgtphi,rgttheta);
   } /* plotpt */


void makeppm(freq)
int freq;
   {
   int i,segs,rownum,strut;
   double lonseg;
   double seglon,seglond,dblsegs;
   double seglend;
   double lat,latd,lat1,lat1d,lat2,lat2d;

   dblsegs = dblfreq;
   segs = freq;  /* segments per row */
   seglend = lon2d / dblfreq;
   lat1d = ninety - heightd;
   lat1 = lat1d * deg2rad;
   rownum = strut = 0;
   while (floor(lat1d) <= ninety)
      {
      lat2 = lat1;
      lat2d = lat2 * rad2deg;
      seglon = curphi[0] = zero;
      curtheta[0] = ninety - lat2d;
      for (i=0;i<segs;i++)
         {
         lonseg = seglend * deg2rad;
         seglon += lonseg;
         seglond = seglon * rad2deg;
         /* get mid-course great circle */
         /* latitude in radians */
         lat = (double) gclat(lat1,lon1,lat2,lon2,seglon);
         latd = lat * rad2deg;
         curphi[i+1] = seglond;
         curtheta[i+1] = ninety - latd;
         if (rownum > 0)
            {
            plotppm(freq,curphi[i],curtheta[i],
               phi[i],theta[i],phi[i+1],theta[i+1]);
            }
         } /* for each segment of the great circle */
      if (rownum > 0 && rownum < freq + 1)
         {
         plotppm(freq,curphi[i],curtheta[i],
            phi[i],theta[i],phi[i+1],theta[i+1]);
         }
      for (i=0;i<=segs;i++)
         {
         phi[i] = curphi[i];
         theta[i] = curtheta[i];
         }
      lat1 += rheight;
      lat1d += rheightd;
      segs--;
      dblsegs--;
      seglend = lon2d / dblsegs;
      rownum++;
      } /* for each row */
   } /* makeppm */

int main(argc,argv)
int argc;
char **argv;
   {
   if (argc != 3) putstx(*argv);
   freq = atoi(*(argv+1)); 
   if (freq < 1) putstx(*argv);
   else if (freq > MAXFREQ)
      {
      fprintf(stderr,"Frequency %d too large\n", freq);
      fprintf(stderr,"Maximum frequency is %d\n", MAXFREQ);
      putstx(*argv);
      }
   diam = atoi(*(argv+2));
   if (diam < 1) putstx(*argv);
   dblfreq  = (double) freq;
   dbldiam  = (double) diam;
   radius   = dbldiam / 2.0;
   phi      = (double *) malloc(sizeof(double)*(freq+32));
   theta    = (double *) malloc(sizeof(double)*(freq+32));
   curphi   = (double *) malloc(sizeof(double)*(freq+32));
   curtheta = (double *) malloc(sizeof(double)*(freq+32));
   strcpy(comma,", ");
   strcpy(newline,"\n");
   zero = 0.0;
   ninety = NINETY;
   pi = PI;
   deg2rad = pi / 180.0;
   rad2deg = 180.0 / pi;
   lon1 = zero;
   lon2d = 72.0;
   lon2 = lon2d * deg2rad;
   height = (double) arclen();
   heightd = height * rad2deg;
   rheight = height / dblfreq; /* row height in radians */
   rheightd = rheight * rad2deg;
   bldhead(freq,diam);
   makeppm(freq);
   printf("</BODY>\n");
   printf("</HTML>\n");
   return(0);
   } /* main */
